//
//  Bus.h
//  HP41
//
//  Created by greg on Wed Aug 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "DataTypes.h"

@class CPU;
@class RomChip;
@class RamChip;
@protocol Peripheral;

@interface Bus : NSObject {
    IBOutlet CPU *cpu;
    RomChip *romChips[0x10]; // [4];  // 16 pages of 4 banks
    RamChip *ramChips[0x40];
    id <Peripheral> peripherals[0x100];
}

- (void) installRomChip: (RomChip *) chip inSlot: (Bits_4) slot;
- (void) installRamChip: (RamChip *) chip inSlot: (Bits_6) slot;
- (void) installPeripheral: (id <Peripheral>) periph inSlot: (Bits_8) slot;

- (BOOL) ramChipInSlot: (Bits_6) slot;

- (BOOL) readRomLocation: (Bits_16) addr into: (Bits_10 *) data;
- (BOOL) writeRomLocation: (Bits_16) addr from: (Bits_10) data;

- (BOOL) readRamLocation: (Bits_4) reg ofChip: (Bits_6) slot into: (Digits_14) data;
- (BOOL) writeRamLocation: (Bits_4) reg ofChip: (Bits_6) slot from: (const Digits_14) data;

- (void) readFromRegister: (Bits_4) reg ofPeripheral: (Bits_8) slot into: (Digits_14) data;
- (void) writeToRegister: (Bits_4) reg ofPeripheral: (Bits_8) slot from: (Digits_14) data;
- (void) writeDataToPeripheral: (Bits_8) slot from: (Digits_14) data;

- (void) abortInstruction: (NSString *) message;

@end
