/* CPU */

#import <Cocoa/Cocoa.h>

#import "CPURegisters.h"

@class Bus;
@class DebugController;
@protocol Display_CPUInterface;

enum PowerMode {
    deepSleep = 0,
    lightSleep = 1,
    powerOn = 2
};

@interface CPU : NSObject
{
    @public
    IBOutlet DebugController *debugController;
    BOOL runFlag;
    BOOL breakpointIsSet;
    Bits_16 breakpointAddr;
    IBOutlet Bus *bus;
    IBOutlet id <Display_CPUInterface> display;
    Bits_16 savedPC;
    Bit nextCarry;
    Bits_2 powerMode;
    BOOL powerOffFlag;
    CPURegisters reg;
    NSTimeInterval simulationTime;
    int keyReleaseDelay;
    int cycleLimit;
}

- (void) reset;
- (void) setPowerMode: (enum PowerMode) mode;
- (BOOL) running;
- (void) setRunning: (BOOL) state;
- (void) step;
- (void) timeSlice: (NSTimer *) timer;
- (BOOL) getBreakpoint: (int *) addr;
- (void) setBreakpointAt: (Bits_16) addr;
- (void) clearBreakpoint;
- (void) unimplementedInstruction;
- (void) abortInstruction: (NSString *) message;

@end
