//
//  PageTabView.m
//  HP41
//
//  Created by greg on Mon Aug 25 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "PageTabView.h"
#import "PDFView.h"
#import "PageTab.h"
#import "DocViewerController.h"

static const float cornerRadius = 10;
static const float textLeftMargin = 5;
static const float tabFontSize = 10;

static NSBezierPath *TabPath(float x0, float y0, float x1, float y1, float r, BOOL closed);

@implementation PageTabView

- (void) awakeFromNib {
    self->font = [[NSFont fontWithName: @"Helvetica" size: tabFontSize] retain];
    NSMutableDictionary *attrs = [NSMutableDictionary dictionary];
    [attrs setObject: self->font forKey: NSFontAttributeName];
    self->textAttrs = [attrs retain];
}

- (void) dealloc {
    [self->font release];
    [self->textAttrs release];
    
    [super dealloc];
}

- (void) setWhichSide: (int) side {
    self->whichSide = side;
}

- (NSArray *) tabs {
    return [self->controller tabs];
}

- (int) numTabs {
    return [[self tabs] count];
}

- (PageTab *) tab: (int) i {
    return [[self tabs] objectAtIndex: i];
}

- (float) tabHeight {
    return ([self bounds].size.height - cornerRadius) / [self numTabs] + cornerRadius;
}

- (NSRect) tabRect: (int) i {
    NSRect bounds = [self bounds];
    float tabSpacing = [self tabHeight] - cornerRadius;
    NSRect r;
    r.origin.x = 0;
    r.origin.y = bounds.size.height - (i + 1) * tabSpacing - cornerRadius;
    r.size.width = bounds.size.width;
    r.size.height = tabSpacing + cornerRadius;
    return r;
}

- (int) tabBeforeInPageOrder: (int) i {
    if (i > 0)
        return i - 1;
    else
        return [self numTabs] - 1;
}

- (int) tabAfterInPageOrder: (int) i {
    if (i < [self numTabs] - 1)
        return i + 1;
    else
        return 0;
}

- (int) tabAboveInStackingOrder: (int) i {
    if (self->whichSide == leftSide)
        return [self tabAfterInPageOrder: i];
    else
        return [self tabBeforeInPageOrder: i];
}

- (int) tabBelowInStackingOrder: (int) i {
     if (self->whichSide == leftSide)
        return [self tabBeforeInPageOrder: i];
    else
        return [self tabAfterInPageOrder: i];
}

- (int) topRhsTabInStackingOrder {
    // Find the topmost tab in the right hand group of tabs. If the
    // current page is a left hand page, first find the corresponding
    // right hand page, then search for the first tab attached to
    // that or a later page.
    int numTabs = [self numTabs];
    int currentRhsPage = [self->controller currentPageForSide: rightSide];
    int i;
    for (i = 0; i < numTabs; i++) {
        PageTab *tab = [self tab: i];
        if (tab->physicalPageNumber >= currentRhsPage)
            return i;
    }
    return 0;
}

- (int) topTabInStackingOrder {
    int i = [self topRhsTabInStackingOrder];
    // The top tab on the left hand side is the one before the top
    // tab on the right hand side.
    if (self->whichSide == leftSide)
        i = [self tabBeforeInPageOrder: i];
    return i;
}

- (int) bottomTabInStackingOrder {
    // Due to wraparound, the bottom tab on either side is the one
    // above the top tab on that side.
    return [self tabAboveInStackingOrder: [self topTabInStackingOrder]];
}

- (BOOL) tabOnThisSide: (PageTab *) tab {
    if ([self->controller allTabsAreTogether])
        return YES;
    else {
        int currentPage = [self->controller currentPageForSide: self->whichSide];
        if (self->whichSide == leftSide) 
            return currentPage >= tab->physicalPageNumber;
        else
            return currentPage <= tab->physicalPageNumber;
    }
}

- (NSBezierPath *) tabPathForRect: (NSRect) r closed: (BOOL) closed {
    if (self->whichSide == leftSide)
        return TabPath(r.origin.x + r.size.width, r.origin.y,
            r.origin.x, r.origin.y + r.size.height, cornerRadius, closed);
    else
        return TabPath(r.origin.x, r.origin.y,
            r.origin.x + r.size.width, r.origin.y + r.size.height, cornerRadius, closed);
}

- (BOOL) isEnabled {
    return [self->controller tabViewIsEnabled: self];
}

- (void) drawRect: (NSRect) rect {
    if ([self isEnabled]) {
        float capHeight = [self->font capHeight];
        //float ascender = [self->font ascender];
        float descender = -[self->font descender];
        float textBase = ([self tabHeight] - capHeight) / 2 - descender;
        //int numTabs = [self numTabs];
        //int i;
        int firstTabNum = [self bottomTabInStackingOrder];
        int tabNum = firstTabNum;
        //for (i = 0; i < numTabs; i++) {
        //    int tabNum = [self tabNumberInStackingOrder: i reverse: NO];
        do {
            PageTab *tab = [self tab: tabNum];
            if ([self tabOnThisSide: tab]) {
                NSRect r = [self tabRect: tabNum];
                NSBezierPath *fillPath = [self tabPathForRect: r closed: YES]; //TabPath(x0, y0, x1, y1, r, YES);
                NSBezierPath *outlinePath = [self tabPathForRect: r closed: NO]; // TabPath(x0, y0, x1, y1, r, NO);
                [[NSColor whiteColor] set];
                [fillPath fill];
                [[NSColor grayColor] set];
                [outlinePath stroke];
                float xt = r.origin.x + textLeftMargin;
                float yt = r.origin.y + textBase;
                [tab->label drawAtPoint: NSMakePoint(xt, yt) withAttributes: self->textAttrs];
            }
            tabNum = [self tabAboveInStackingOrder: tabNum];
        } while (tabNum != firstTabNum);
    }
}

- (NSView *) hitTest: (NSPoint) p {
    if ([self isEnabled])
        return [super hitTest: p];
    else
        return nil;
}

- (void) mouseDown: (NSEvent *) event {
    NSPoint p = [self convertPoint: [event locationInWindow] fromView: nil];
    int firstTabNum = [self topTabInStackingOrder];
    int tabNum = firstTabNum;
    //int numTabs = [self numTabs];
    //int i;
    //for (i = 0; i < numTabs; i++) {
    //    int tabNum = [self tabNumberInStackingOrder: i reverse: YES];
    do {
        PageTab *tab = [self tab: tabNum];
        if ([self tabOnThisSide: tab]) {
            NSRect r = [self tabRect: tabNum];
            if (NSMouseInRect(p, r, NO)) {
                [self->controller goToPage: tab->physicalPageNumber];
                return;
            }
        }
        tabNum = [self tabBelowInStackingOrder: tabNum];
    } while (tabNum != firstTabNum);
    if (self->whichSide == 0)
        [self->controller goToPage: 0];
    else
        [self->controller goToPage: [self->controller numPages]];
}
    
@end

static NSBezierPath *TabPath(float x0, float y0, float x1, float y1, float r, BOOL closed) {
    NSBezierPath *path = [NSBezierPath bezierPath];
    [path moveToPoint: NSMakePoint(x0, y0)];
    [path appendBezierPathWithArcFromPoint: NSMakePoint(x1, y0)
        toPoint: NSMakePoint(x1, y1) radius: r];
    [path appendBezierPathWithArcFromPoint: NSMakePoint(x1, y1)
        toPoint: NSMakePoint(x0, y1) radius: r];
    [path lineToPoint: NSMakePoint(x0, y1)];
    if (closed)
        [path closePath];
    return path;
}
