//
//  RomDisplay.m
//  HP41
//
//  Created by greg on Wed Aug 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "RomDisplay.h"
#import "DebugController.h"
#import "Disassembly.h"

static float lineHeight = 16;
static float leftMargin = 4;
static float markerGap = 4;

static BOOL DisassembleWindow(int numberMode, int pc0, int pc1, int pc, int words[], 
    NSString *prefixes[], NSString *suffixes[])
{
    int addr = pc0;
    BOOL hitPc = NO;
    int i, nwords;
    
    for (i = 0; i <= pc1 - pc0; i++) {
        prefixes[i] = suffixes[i] = nil;
    }
    while (addr <= pc1) {
        i = addr - pc0;
        if (addr == pc)
            hitPc = YES;
        if (words[i] >= 0 && words[i+1] >= 0)
            DisassembleInstruction(numberMode, addr, words[i], words[i+1], 
                &prefixes[i], &suffixes[i], &nwords);
        else {
            prefixes[i] = @"";
            suffixes[i] = @"";
            nwords = 1;
        }
        addr += nwords;
    }
    return hitPc;
}

@implementation RomDisplay

- (IBAction) numberModeControlChanged: (id) sender {
    self->numberMode = [sender selectedColumn];
    [self setNeedsDisplay: YES];
}

- (void) drawRect: (NSRect) rect {
    NSRect bounds;
    int numLines, charWidth, pc, pc0, pc1, addr, data, breakpoint;
    float x;
    int y;
    NSString *s, *pcMarker, *bpMarker, **prefixes, **suffixes;
    int i, *words;
    
    bounds = [self bounds];
    [[NSColor blackColor] set];
    NSRectFill(bounds);
    if ([self->debugController running])
        return;
    numLines = bounds.size.height / lineHeight;
    if (!(numLines & 1))
        numLines -= 1;
    pcMarker = @">";
    bpMarker = @"*";
    //markerWidth = [marker sizeWithAttributes: self->attributes].width;
    charWidth = [@"X" sizeWithAttributes: self->attributes].width;
    pc = [self->debugController pc];
    if (![self->debugController getBreakpoint: &breakpoint])
        breakpoint = -1;
    pc0 = pc - numLines / 2;
    pc1 = pc + numLines / 2;
    words = malloc((numLines + 1) * sizeof(int));
    prefixes = calloc(1, numLines * sizeof(NSString *));
    suffixes = calloc(1, numLines * sizeof(NSString *));
    for (addr = pc0; addr <= pc1 + 1; addr++) {
        i = addr - pc0;
        if (!(addr >= 0 && addr <= 0xffff) || 
            ![self->debugController readRomLocation: addr into: &words[i]])
                words[i] = -1;
    }
    if (!DisassembleWindow(self->numberMode, pc0, pc1, pc, words, prefixes, suffixes))
        DisassembleWindow(self->numberMode, pc0 + 1, pc1, pc, words, prefixes, suffixes);
    //x = leftMargin + markerWidth + markerGap;
    y = bounds.size.height - (lineHeight - self->digitSize.height) / 2 - lineHeight;
    for (addr = pc0; addr <= pc1; addr++) {
        if (addr >= 0 && addr <= 0xffff) {
            i = addr - pc0;
            data = words[i];
            if (addr == pc) {
                x = leftMargin;
                [pcMarker drawAtPoint: NSMakePoint(x, y) withAttributes: self->attributes];
            }
            NSString *addrString = FormatRomAddressUsingMode(self->numberMode, addr);
            if (data >= 0) {
                NSString *dataString = FormatNumberUsingMode(self->numberMode, data, 10);
                s = [NSString stringWithFormat: @"%@: %@", addrString, dataString];
            }
            else
                s = addrString;
            x = leftMargin + charWidth + markerGap;
            [s drawAtPoint: NSMakePoint(x, y) withAttributes: self->attributes];
            if (addr == breakpoint)
                [bpMarker drawAtPoint: NSMakePoint(x + 15 * charWidth, y) withAttributes: self->attributes];
            [prefixes[i] drawAtPoint: NSMakePoint(x + 17 * charWidth, y) withAttributes: self->attributes];
            [suffixes[i] drawAtPoint: NSMakePoint(x + 27 * charWidth, y) withAttributes: self->attributes];
        }
        y -= lineHeight;
    }
    free(words);
    free(prefixes);
    free(suffixes);
}

@end
