//
//  Disassembly.m
//  HP41
//
//  Created by greg on Wed Aug 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "Disassembly.h"

struct MiscDesc {
    int opcode;
    NSString *prefix;
    NSString *suffix;
};

struct Class0Desc {
    NSString *prefix;
    char paramType;
};

struct FDesc {
    int value;
    char *code;
};

static struct MiscDesc miscTable[] = {
    {0x3C4, @"ST=0"},
    {0x3C8, @"CLRKEY"},
    {0x3CC, @"?KEY"},
    {0x3D4, @"R=R-1"},
    {0x3DC, @"R=R+1"},
    {0x058, @"G=C"},
    {0x098, @"C=G"},
    {0x0D8, @"C<>G"},
    {0x158, @"M=C"},
    {0x198, @"C=M"},
    {0x1D8, @"C<>M"},
    {0x258, @"T=ST"},
    {0x298, @"ST=T"},
    {0x2D8, @"ST<>T"},
    {0x358, @"ST=C"},
    {0x398, @"C=ST"},
    
    {0x020, @"XQ>G0"},
    {0x060, @"POWOFF"},
    {0x0A0, @"SLCT P"},
    {0x0E0, @"SLCT Q"},
    {0x120, @"?P=Q"},
    {0x160, @"?LOWBAT"},
    {0x1A0, @"A=B=C=0"},
    {0x1E0, @"GOTO ADR"},
    {0x220, @"C=KEY"},
    {0x260, @"SETHEX"},
    {0x2A0, @"SETDEC"},
    {0x2E0, @"DSPOFF"},
    {0x320, @"DSPTOG"},
    {0x360, @"C RTN"},
    {0x3A0, @"NC RTN"},
    {0x3D8, @"C<>ST"},
    
    {0x070, @"N=C"},
    {0x0B0, @"C=N"},
    {0x0F0, @"C<>N"},
    {0x130, @"LDI S&X"},
    {0x170, @"PUSH ADR"},
    {0x1B0, @"POP ADR"},
    {0x230, @"GOTO KEY"},
    {0x270, @"RAM SLCT"},
    {0x2F0, @"WRITE DATA"},
    {0x038, @"READ DATA"},
    {0x330, @"FETCH S&X"},
    {0x370, @"C=C OR A"},
    {0x3B0, @"C=C AND A"},
    {0x3F0, @"PRPH SLCT"},
    {0x3E0, @"RTN"},
};

static struct Class0Desc class0table[] = {
    /* 0 */ {@"NOP",   '-'},
    /* 1 */ {@"CLRF",  'f'},
    /* 2 */ {@"SETF",  'f'},
    /* 3 */ {@"?FSET", 'f'},
    /* 4 */ {@"LD@R-", 'd'},
    /* 5 */ {@"?R=",   'f'},
    /* 6 */ {nil,      '-'},
    /* 7 */ {@"R=",    'f'},
    /* 8 */ {nil,      '-'},
    /* 9 */ {@"SELP",  'f'},
    /* A */ {@"WRITE", 'r'},
    /* B */ {@"?FI",   'f'},
    /* C */ {nil,      '-'},
    /* D */ {nil,      '-'},
    /* E */ {@"READ",  's'},
    /* F */ {@"RCR",   'f'},
};

static struct FDesc fTable[] = {
    /* 0 */ {0x3, "T"},
    /* 1 */ {0x4, "Z"},
    /* 2 */ {0x5, "Y"},
    /* 3 */ {0xA, "X"},
    /* 4 */ {0x8, "L"},
    /* 5 */ {0x6, "M"},
    /* 6 */ {0xB, "N"},
    /* 7 */ {0xE, "O"},
    /* 8 */ {0x2, "P"},
    /* 9 */ {0x9, "Q"},
    /* A */ {0x7, "|-"},
    /* B */ {0xD, "a"},
    /* C */ {0x1, "b"},
    /* D */ {0xC, "c"},
    /* E */ {0x0, "d"},
    /* F */ {0xF, "e"},
};

static NSString *class1table[] = {
    @"NCX", @"CX", @"NCG", @"CG"
};

static NSString *class2opcodeTable[] = {
    @"A=0",
    @"B=0",
    @"C=0",
    @"A<>B",
    @"B=A",
    @"A<>C",
    @"C=B",
    @"C<>B",
    @"A=C",
    @"A=A+B",
    @"A=A+C",
    @"A=A+1",
    @"A=A-B",
    @"A=A-1",
    @"A=A-C",
    @"C=C+C",
    @"C=C+A",
    @"C=C+1",
    @"C=A-C",
    @"C=C-1",
    @"C=0-C",
    @"C=-C-1",
    @"?B!=0",
    @"?C!=0",
    @"?A<C",
    @"?A<B",
    @"?A!=0",
    @"?A!=C",
    @"RSHFA",
    @"RSHFB",
    @"RSHFC",
    @"LSHFA",
};

static NSString *class2fieldTable[] = {
    @"@R",
    @"S&X",
    @"R<",
    @"ALL",
    @"P-Q",
    @"XS",
    @"M",
    @"MS",
};

static NSString *class3table[] = {
    @"JNC", @"JC"
};

static int gNumberMode;

NSString *FormatNumberUsingMode(int numberMode, int value, int bits) {
    switch (numberMode) {
        case disasmHex:
            return [NSString stringWithFormat: @"%0*X", (bits + 3) / 4, value];
        case disasmOctal:
            return [NSString stringWithFormat: @"%0*o", (bits + 2) / 3, value];
    }
    return nil; // not reached
}

static NSString *FormatNumber(int value, int bits) {
    return FormatNumberUsingMode(gNumberMode, value, bits);
}

NSString *FormatRomAddressUsingMode(int numberMode, int addr) {
    switch (numberMode) {
        case disasmHex:
            return FormatNumberUsingMode(disasmHex, addr, 16);
            break;
        case disasmOctal:
            return [NSString stringWithFormat: @"%3o,%04o", (addr >> 10) << 1, addr & 0x3ff];
            break;
    }
    return nil; // not reached
}

static void FormatRomAddress(int addr, NSString **result) {
    *result = FormatRomAddressUsingMode(gNumberMode, addr);
}

static void DisassembleMisc(int word1, NSString **prefix, NSString **suffix) {
    int i, n;
    struct MiscDesc *p;
    n = sizeof(miscTable) / sizeof(miscTable[0]);
    for (i = 0; i < n; i++) {
        p = &miscTable[i];
        if (p->opcode == word1) {
            *prefix = p->prefix;
            *suffix = p->suffix;
            return;
        }
    }
    *prefix = @"????";
    *suffix = @"";
}

static void DisassembleClass0(int word1, NSString **prefix, NSString **suffix) {
    int param = word1 >> 6;
    int opcode = (word1 >> 2) & 0xf;
    struct Class0Desc *d = &class0table[opcode];
    struct FDesc *f = &fTable[param];
    if (d->paramType == 'f')
        param = f->value;
    if (!d->prefix || 
        (d->paramType == 'f' && param > 13) || 
        (d->paramType == 's' && param < 1))
            DisassembleMisc(word1, prefix, suffix);
    else {
        *prefix = d->prefix;
        if (d->paramType == 'r' || d->paramType == 's')
            //*suffix = [NSString stringWithFormat: @"%X(%s)", param, f->code];
            *suffix = [NSString stringWithFormat: @"%@(%s)", FormatNumber(param, 4), f->code];
        else
            //*suffix = [NSString stringWithFormat: @"%X", param];
            *suffix = FormatNumber(param, 4);
    }
}

static void DisassembleClass1(int word1, int word2, NSString **prefix, NSString **suffix) {
    int addr = (word2 >> 2) << 8 | (word1 >> 2);
    int uv = word2 & 3;
    *prefix = class1table[uv];
    FormatRomAddress(addr, suffix);
}

static void DisassembleClass2(int word1, NSString **prefix, NSString **suffix) {
    int opcode = word1 >> 5;
    int field = (word1 >> 2) & 7;
    *prefix = class2opcodeTable[opcode];
    *suffix = class2fieldTable[field];
}

static void DisassembleClass3(int pc, int word1, NSString **prefix, NSString **suffix) {
    int offset = word1 >> 3;
    int v = (word1 >> 2) & 1;
    if (offset >= 64)
        offset -= 128;
    *prefix = class3table[v];
    FormatRomAddress(pc + offset, suffix);
}

void DisassembleInstruction(int numberMode, int pc, int word1, int word2, 
    NSString **prefix, NSString **suffix, int *nwords) 
{
    gNumberMode = numberMode;
    if (word1 == 0x130) {
        *prefix = @"LDI S&X";
        //*suffix = [NSString stringWithFormat: @"%03X", word2];
        *suffix = FormatNumber(word2, 10);
        *nwords = 2;
    }
    else switch (word1 & 0x3) {
        case 0:
            DisassembleClass0(word1, prefix, suffix);
            *nwords = 1;
            break;
        case 1:
            DisassembleClass1(word1, word2, prefix, suffix);
            *nwords = 2;
            break;
        case 2:
             DisassembleClass2(word1, prefix, suffix);
             *nwords = 1;
            break;
        case 3:
            DisassembleClass3(pc, word1, prefix, suffix);
            *nwords = 1;
            break;
    }
}
